import re

from ao.social.oauth_ import OAuthClient
from ao.social.utils import urlfetch

from oauth import oauth

from xml.dom import minidom


rx_key = re.compile('<url>.*?\?.*?;key=(.*?)(&.*|</url>)')


class XML(object):
    """Simple one-level XML node getter"""

    def __init__(self, data):
        """Parse the DOM."""

        self._xml = minidom.parseString(data)

    def __getitem__(self, name):
        """Return the value of the named text node."""

        return self._xml.getElementsByTagName(name)[0].childNodes[0].nodeValue


class LinkedInClient(OAuthClient):
    """LinkedIn-specific OAuth client."""

    _default_config = {
        'request_token_url': 'https://api.linkedin.com/uas/oauth/requestToken',
        'access_token_url': 'https://api.linkedin.com/uas/oauth/accessToken',
        'authorize_url': 'https://api.linkedin.com/uas/oauth/authorize',
        'account_verification_url':
            'https://api.linkedin.com/v1/people/~',
        'update_url': 'http://api.linkedin.com/v1/people/~/current-status',
    }
    _set_status = '<?xml version="1.0" encoding="UTF-8"?><current-status>'\
        '%(status)s</current-status>'

    def lookup_user_info(self, access_token, access_secret):
        """Lookup User Info.

        Same as `get_user_info` except that it uses a stored access_token and
        access_secret.

        """

        resp = self._make_protected_request(access_token, access_secret, True)
        xml = XML(resp.content)

        return {
            'id': rx_key.search(resp.content).group(1),
            'first_name': xml['first-name'],
            'last_name': xml['last-name'],
        }

    def post(self, text, token='', secret=''):
        """Do a LinkedIn profile update."""

        url = self._config['update_url']

        data = self._set_status % {
            'status': text.encode('utf-8'),
        }

        token = oauth.OAuthToken(token, secret)
        request = oauth.OAuthRequest.from_consumer_and_token(
            self._consumer,
            token=token,
            http_url=url,
            http_method='PUT',
        )
        request.sign_request(
            self._signature_method,
            self._consumer,
            token,
        )

        urlfetch.fetch(
            url,
            payload=data,
            method=urlfetch.PUT,
            headers=request.to_header(),
        )



import webob
import webob.exc

from ao import social

from oauth import oauth


class AuthMiddleware(object):
    """Authentication and authorization middleware."""

    _popup_html = """
<!doctype html>
<html lang="en">
  <head>
    <meta charset="utf-8"/>
    <title></title>
    <script>
      try {
        %(postlogin)s;
      } catch (e) {};
      close();
    </script>
  </head>
</html>"""

    def __init__(self, app, config={}):
        """Configure the middleware."""

        self._app = app

        self._user_class = self._import_user(config['user_class'])
        self._login_path = config['login_path']

        self._clients = {}
        for method in ('facebook', 'google', 'twitter', 'linkedin'):
            if method in config:
                self._clients[method] = social.registerClient(method,
                    config[method])

    def __call__(self, environ, start_response):
        """Put the user object into the WSGI environment."""

        # Create our own request object
        request = webob.Request(environ, charset='utf-8')

        # Save the login path, we might require it in template tags
        environ['ao.social.login'] = self._build_absolute_uri(environ,
            self._login_path)

        # We need beaker sessions for this middleware
        session = environ['beaker.session']

        # Check if the user already has a session
        environ['ao.social.user'] = None
        if 'ao.social.user' in session:
            environ['ao.social.user'] = self._user_class.get_user(
                session['ao.social.user'])

        # Try to log in the user
        for method in ('facebook', 'twitter', 'google', 'linkedin'):
            if request.path_info == self._login_path % method:
                response = self._handle_user(request, method, 'login')
                if response is not None:
                    return response(environ, start_response)

        # Call the downstream application
        return self._app(environ, start_response)

    def _build_absolute_uri(environ, path='/'):
        """Constructs an absolute URI."""

        root = '%s://%s' % (environ['wsgi.url_scheme'], environ['HTTP_HOST'])
        path = not path.startswith('/') and environ['PATH_INFO'] + path or path

        return root + path

    _build_absolute_uri = staticmethod(_build_absolute_uri)

    def _import_user(cls):
        """Import the provided `User` class."""

        modstr, _, cls = cls.rpartition('.')

        mod = __import__(modstr)
        for sub in modstr.split('.')[1:]:
            mod = getattr(mod, sub)

        return getattr(mod, cls)

    _import_user = staticmethod(_import_user)

    def _login_user(self, request, method, credentials):
        """Looks up the user and initiates a session."""

        id = str(credentials['id'])
        uid = ':'.join((method, id))
        session = request.environ['beaker.session']

        # Get the user from the database backend (or create a new user)
        user = self._user_class.lookup_user(uid)

        # Save the user's details and any associated tokens
        if method == 'facebook':
            info = ['name', 'first_name', 'last_name', 'email', 'pic_square']
            data = self._clients[method].users.getInfo(id, info)[0]
            user.update_details({
                'name': data['name'],
                'first_name': data['first_name'],
                'last_name': data['last_name'],
                'avatar': data['pic_square'],
                'email': data['email'],
            })
            user.set_token('facebook', {
                'uid': id,
                'token': credentials['token'],
                'secret': credentials['secret'],
            })
        elif method == 'twitter':
            first_name, _, last_name = credentials['name'].partition(' ')
            user.update_details({
                'name': credentials['name'],
                'first_name': first_name,
                'last_name': last_name,
                'avatar': credentials['profile_image_url'],
            })
            user.set_token('twitter', {
                'uid': id,
                'token': credentials['token'],
                'secret': credentials['secret'],
            })
        elif method == 'google':
            user.update_details({
                'name': '%s %s' % (credentials['first_name'],
                    credentials['last_name']),
                'first_name': credentials['first_name'],
                'last_name': credentials['last_name'],
                'email': credentials['email'],
            })
            user.set_token('google', {
                'uid': id,
            })
        elif method == 'linkedin':
            user.update_details({
                'name': '%s %s' % (credentials['first_name'],
                    credentials['last_name']),
                'first_name': credentials['first_name'],
                'last_name': credentials['last_name'],
            })
            user.set_token('linkedin', {
                'uid': id,
                'token': credentials['token'],
                'secret': credentials['secret'],
            })

        # Prepare the response
        postlogin = ''
        if 'postlogin' in session:
            postlogin = session['postlogin']
            del session['postlogin']
        body = self._popup_html % {'postlogin': postlogin}
        response = webob.Response(body=body)

        # Store the user's key in the session
        session['ao.social.user'] = str(user.get_key())
        session.save()

        # Save changes to the user object
        user.save_user()

        return response

    def _connect_user(self, request, method, credentials):
        """Connects the account to the current user."""

        id = str(credentials['id'])
        user = request.environ['ao.social.user']
        session = request.environ['beaker.session']

        # Save the user's details and any associated tokens
        if method == 'facebook':
            user.set_token('facebook', {
                'uid': id,
                'token': credentials['token'],
                'secret': credentials['secret'],
            })
        elif method == 'twitter':
            user.set_token('twitter', {
                'uid': id,
                'token': credentials['token'],
                'secret': credentials['secret'],
            })
        elif method == 'google':
            user.set_token('google', {
                'uid': id,
            })
        elif method == 'linkedin':
            user.set_token('linkedin', {
                'uid': id,
                'token': credentials['token'],
                'secret': credentials['secret'],
            })

        # Prepare the response
        postlogin = ''
        if 'postlogin' in session:
            postlogin = session['postlogin']
            del session['postlogin']
        body = self._popup_html % {'postlogin': postlogin}
        response = webob.Response(body=body)

        # Save changes to the user object
        user.save_user()

        return response

    def _handle_user(self, request, method, mode='login'):
        """Handles authentication for the user.

        If `mode` is set to 'connect', it will assume that a user is already
        logged in and connects the new account to the logged in user.
        Otherwise, simply logs in the user.

        """

        user = request.environ['ao.social.user']
        session = request.environ['beaker.session']

        # Save the post login action in the session.
        if 'postlogin' in request.GET:
            session['postlogin'] = request.GET['postlogin']
            session.save()

        # Check if the user has logged in via Facebook Connect.
        if method == 'facebook':
            facebook_user = self._clients[method].get_user(request)
            if facebook_user is None:
                raise social.Unauthorized('Facebook Connect authentication '\
                    'failed.')
            # OK, Facebook user is verified.
            if user is None:
                return self._login_user(request, method, facebook_user)
            return self._connect_user(request, method, facebook_user)

        # Check if the user has logged in via Twitter's Oauth.
        if method == 'twitter':
            keys = ('oauth_token', 'oauth_verifier')
            if  not all(key in request.GET for key in keys):
                # Redirect the user to Twitter's authorization URL.
                auth_url = self._login_path % method
                auth_url = self._clients[method].get_authorization_url(
                    self._build_absolute_uri(request.environ, auth_url))
                return webob.Response(status=302, headers={
                    'Location': auth_url,
                })
            try:
                twitter_user = self._clients[method].get_user_info(
                    request.GET['oauth_token'],
                    request.GET['oauth_verifier'],
                )
                # OK, Twitter user is verified.
                if user is None:
                    return self._login_user(request, method, twitter_user)
                return self._connect_user(request, method, twitter_user)
            except oauth.OAuthError:
                raise social.Unauthorized('Twitter OAuth authentication '\
                    'failed.')

        # Check if the user has logged in via Google OpenID/OAuth.
        if method == 'google':
            if len(request.GET) < 2:
                # Create a custom auth request and redirect the user.
                return webob.exc.HTTPTemporaryRedirect(
                    location=self._clients[method].redirect(),
                )
            # Hopefully the user has come back from the auth request url.
            callback = self._build_absolute_uri(request.environ,
                self._login_path % method)
            google_user = self._clients[method].get_user(request.GET, callback)
            if google_user is None:
                raise social.Unauthorized('Google OpenID authentication '\
                    'failed.')
            # OK, Google user is verified.
            if user is None:
                return self._login_user(request, method, google_user)
            return self._connect_user(request, method, google_user)

        # Check if the user has logged in via LinkedIn's Oauth.
        if method == 'linkedin':
            keys = ('oauth_token', 'oauth_verifier')
            if  not all(key in request.GET for key in keys):
                # Redirect the user to LinkedIn's authorization URL.
                auth_url = self._login_path % method
                auth_url = self._clients[method].get_authorization_url(
                    self._build_absolute_uri(request.environ, auth_url))
                return webob.Response(status=302, headers={
                    'Location': auth_url,
                })
            try:
                linkedin_user = self._clients[method].get_user_info(
                    request.GET['oauth_token'],
                    request.GET['oauth_verifier'],
                )
                # OK, LinkedIn user is verified.
                if user is None:
                    return self._login_user(request, method, linkedin_user)
                return self._connect_user(request, method, linkedin_user)
            except oauth.OAuthError:
                raise social.Unauthorized('LinkedIn OAuth authentication '\
                    'failed.')



try:
    import json

except ImportError:
    try:
        import simplejson as json

    except ImportError:
        try:
            from django.utils import simplejson as json

        except ImportError:
            raise ImportError('One of the following modules must be '\
                'available: `json`, `simplejson`, `django.utils.simplejson`.')

try:
    from google.appengine.api import memcache

except ImportError:
    try:
        import memcache as memcache_module
        mc = memcache_module.Client(['127.0.0.1:11211'], debug=0)

        class MemCache(object):
            """Mock the memcache object."""

            def get(self, key):
                """Get the key from the cache."""
                #FIXME: mmh, probably not so good
                return mc.get(key.encode('utf-8'))

            def set(self, key, value, time):
                """Get the key from the cache."""
                mc.set(key, value, time)

        memcache = MemCache()

    except ImportError:
        raise ImportError('You must install memcache')

try:
    from google.appengine.api import urlfetch

except ImportError:
    import urllib2

    class URLFetch(object):
        """Fall-back URL fetch."""

        GET, POST = 'GET', 'POST'

        def fetch(self, url, payload=None, method=None, headers={}):
            """Somulate App Engine's `fetch` method for this module."""

            method = method or self.GET
            if method == self.GET:
                payload = None
            if method == self.POST:
                payload = payload or ''

            request = urllib2.Request(url, data=payload, headers=headers)

            return Response(request)

    class Response(object):
        """Simulate App Engine's URL fetch response."""

        def __init__(self, request):
            """Store the request object."""

            self._request = request
            self._response = None

        def _fetch(self):
            """Lazily fetch the response."""

            if self._response is None:
                response = urllib2.urlopen(self._request)
                self._response = {
                    'obj': response,
                    'data': response.read(),
                }

        @property
        def status_code(self):
            return self._response['obj'].getcode()

        def content(self):
            """Simulate the content property."""

            self._fetch()
            return self._response['data']

        content = property(content)

    urlfetch = URLFetch()




from ao.social import facebook_ as facebook
from ao.social import twitter_ as twitter
from ao.social import google_ as google
from ao.social import linkedin_ as linkedin
from ao.social.utils import json


clients = {}


class Unauthorized(Exception):
    """User is not authorized."""


class ImproperlyConfigured(Exception):
    """Indicates that the short url handler is not configured properly."""


def registerClient(method, config={}):
    """Register a Twitter client."""

    client_classes = {
        'twitter': twitter.TwitterClient,
        'google': google.GoogleClient,
        'facebook': facebook.FacebookClient,
        'linkedin': linkedin.LinkedInClient,
    }

    global clients
    clients[method] = client_classes[method](config)

    return clients[method]


def getClient(method):
    """Return the client for the given method."""

    global clients

    try:
        return clients[method]
    except KeyError:
        raise ImproperlyConfigured('The requested client (%s) is not '\
            'initialized (available clients: %s).' % (method, clients.keys()))


def user(request):
    """Adds the session user to the `RequestContext`.

    This is a Django `template context processor`. You shouldn't need to use
    this function directly.

    """

    return {
        'user': request.environ['ao.social.user'],
    }


class UserBase(object):
    """Base class for `User` implementations."""

    def post(self, method, text, *args, **kw):
        """Do a stream post or status update to the given service."""

        token = self.get_token(method)
        if method == 'facebook':
            kw['uid'] = token['uid']
        elif method in ('twitter', 'linkedin'):
            kw['token'] = token['token']
            kw['secret'] = token['secret']

        client = getClient(method)

        return client.post(text, *args, **kw)

    def get_user(cls, key):
        """Get a user for the corresponding method & external id."""

        raise NotImplementedError('You must overload the `get_user` method.')

    get_user = classmethod(get_user)

    def lookup_user(cls, uid):
        """Get a user for the corresponding method & external id."""

        raise NotImplementedError('You must overload the `lookup_user` '\
            'method.')

    lookup_user = classmethod(lookup_user)

    def save_user(self):
        """Get a user for the corresponding method & external id."""

        raise NotImplementedError('You must overload the `save_user` method.')

    def get_key(self):
        """Return a key that will be stored in session."""

        raise NotImplementedError('You must overload the `get_key` method.')

    def update_details(self, details):
        """Update the user's details."""

        raise NotImplementedError('You must overload the `update_details` '\
            'method.')

    def id(self):
        """Return the login id."""

        return self.uid.partition(':')[2]

    id = property(id)

    def method(self):
        """Return the login method ('google', 'twitter' or 'facebook')."""

        return self.uid.partition(':')[0]

    method = property(method)

    def get_token(self, method):
        """Return the token for the given method."""

        if not hasattr(self, 'tokens'):
            raise NotImplementedError('User object has no `tokens` attribute.')

        return json.loads(self.tokens or '{}')[method]

    def set_token(self, method, token=None):
        """Store the token for the user."""

        if not hasattr(self, 'tokens'):
            raise NotImplementedError('User object has no `tokens` attribute.')

        tokens = json.loads(self.tokens or '{}')
        if token is None:
            del tokens[method]
        else:
            tokens[method] = token

        self.tokens = json.dumps(tokens, separators=(',', ':'))

        self.save_user()

    def clear_token(self, method):
        """Remove the token."""

        return self.set_token(method)



import cgi

from ao.social.utils import memcache, urlfetch

from oauth import oauth

import urllib


class OAuthClient(object):
    """Generic OAuth client."""

    _default_config = {
        'request_token_url': '',
        'access_token_url': '',
        'authorize_url': '',
        'account_verification_url': '',
        'update_url': '',
    }

    def __init__(self, config={}):
        """Configure the client."""

        for key, value in self._default_config.iteritems():
            if key not in config:
                config[key] = value

        self._config = config

        self._signature_method = oauth.OAuthSignatureMethod_HMAC_SHA1()
        self._consumer = oauth.OAuthConsumer(config['key'], config['secret'])

    def key(self):
        """Return the API key."""

        return self._config['key']

    def _extract_credentials(self, result):
        """Extract Credentials.

        Returns an dictionary containing the token and secret (if present),
        or raises OAuthError.

        """

        token = None
        secret = None
        parsed_results = cgi.parse_qs(result.content)

        if 'oauth_token' in parsed_results:
            token = parsed_results['oauth_token'][0]

        if 'oauth_token_secret' in parsed_results:
            secret = parsed_results['oauth_token_secret'][0]

        if not (token and secret) or result.status_code != 200:
            raise oauth.OAuthError, 'Problem talking to the service'

        return {
            'token': token,
            'secret': secret,
        }

    def _get_memcache_auth_key(self, auth_token):
        """Get the memcache auth_key from an auth_token."""

        return 'oauth_%s' % auth_token

    def _get_auth_token(self, callback_url):
        """Get Authorization Token.

        Actually gets the authorization token and secret from the service. The
        token and secret are stored in memcache, and the auth token is
        returned.

        """

        # Create an OAuth request and extract the credentials.
        result = self._extract_credentials(
            self._make_callback_request(callback_url))

        auth_token = result['token']
        auth_secret = result['secret']

        # Add the secret to memcache.
        memcache.set(self._get_memcache_auth_key(auth_token), auth_secret,
            time=1200)

        return auth_token

    def _make_callback_request(self, callback_url):
        """Makes a generic OAuth request."""

        request = oauth.OAuthRequest.from_consumer_and_token(
            self._consumer,
            callback=callback_url,
            http_url=self._config['request_token_url'],
        )
        request.sign_request(
            self._signature_method,
            self._consumer,
            None,
        )

        return urlfetch.fetch(request.to_url())

    def _make_verification_request(self, token, secret, verifier, params):
        """Makes a verification OAuth request."""

        token = oauth.OAuthToken(token, secret)
        request = oauth.OAuthRequest.from_consumer_and_token(
            self._consumer,
            token=token,
            verifier=verifier,
            http_url=self._config['access_token_url'],
        )
        request.sign_request(
            self._signature_method,
            self._consumer,
            token,
        )

        return urlfetch.fetch(request.to_url())

    def _make_protected_request(self, token, secret, headers=False):
        """Makes a protected OAuth request object."""

        url = self._config['account_verification_url']
        token = oauth.OAuthToken(token, secret)
        request = oauth.OAuthRequest.from_consumer_and_token(
            self._consumer,
            token=token,
            http_url=url,
        )
        request.sign_request(
            self._signature_method,
            self._consumer,
            token,
        )

        if headers:
            # LinkedIn requires the parameters to be sent as headers
            return urlfetch.fetch(url, headers=request.to_header())

        # Twitter works fine with URL query string parameters
        return urlfetch.fetch(request.to_url(),
            headers={'Authorization': 'OAuth'})

    def get_authorization_url(self, callback_url):
        """Get Authorization URL."""

        return '%s?oauth_token=%s' % (self._config['authorize_url'],
            self._get_auth_token(callback_url))

    def get_user_info(self, auth_token, auth_verifier):
        """Get User Info.

        Exchanges the auth token for an access token and returns a dictionary
        of information about the authenticated user.

        """

        auth_token = urllib.unquote(auth_token)
        auth_verifier = urllib.unquote(auth_verifier)

        auth_secret = memcache.get(self._get_memcache_auth_key(auth_token))
        auth_secret = auth_secret or ''  # memcache might return None

        response = self._make_verification_request(auth_token, auth_secret,
            auth_verifier, {'oauth_verifier': auth_verifier})

        # Extract the access token/secret from the response.
        result = self._extract_credentials(response)

        # Try to collect some information about this user from the service.
        user_info = self.lookup_user_info(result['token'], result['secret'])
        user_info.update(result)

        return user_info

    def lookup_user_info(self, access_token, access_secret):
        """Lookup User Info.

        Same as `get_user_info` except that it uses a stored access_token and
        access_secret.

        """

        raise NotImplementedError('Subclasses must implement this method.')

    def post(self, text, token='', secret=''):
        """Do a profile profile update."""

        raise NotImplementedError('Subclasses must implement this method.')



import urllib

from ao.social.oauth_ import OAuthClient
from ao.social.utils import json, urlfetch

from oauth import oauth


class TwitterClient(OAuthClient):
    """Twitter-specific OAuth client."""

    _default_config = {
        'request_token_url': 'https://twitter.com/oauth/request_token',
        'access_token_url': 'https://twitter.com/oauth/access_token',
        'authorize_url': 'https://twitter.com/oauth/authorize',
        'account_verification_url':
            'https://twitter.com/account/verify_credentials.json',
        'update_url': 'https://api.twitter.com/1/statuses/update.%(format)s',
    }

    def lookup_user_info(self, access_token, access_secret):
        """Lookup User Info.

        Same as `get_user_info` except that it uses a stored access_token and
        access_secret.

        """

        response = self._make_protected_request(access_token, access_secret)

        return json.loads(response.content)

    def post(self, text, token='', secret=''):
        """Do a Twitter profile update."""

        url = self._config['update_url'] % {
            'format': 'json',
        }

        data = {
            'status': text.encode('utf-8'),
        }
        data = urllib.urlencode(data)

        token = oauth.OAuthToken(token, secret)
        request = oauth.OAuthRequest.from_consumer_and_token(
            self._consumer,
            token=token,
            http_url=url,
            http_method='POST',
        )
        request.set_parameter('status', text)
        request.sign_request(
            self._signature_method,
            self._consumer,
            token,
        )

        response = urlfetch.fetch(
            url,
            payload=data,
            method=urlfetch.POST,
            headers=request.to_header(),
        )

        return json.loads(response.content)



import facebook

from hashlib import md5


class FacebookClient(object):
    """Facebook Connect handler."""

    def __init__(self, config={}):
        """Configure the client."""

        self._key = config['key']
        self._secret = config['secret']

        self._api = facebook.Facebook(self._key, self._secret)

    def key(self):
        """Return the API key."""

        return self._key

    def get_user(self, request):
        """Check if the user has authorized the application."""

        if self._key in request.cookies:
            hash = md5('expires=%ssession_key=%sss=%suser=%s%s' % tuple(
                [request.cookies.get('%s_%s' % (self._key, item)) \
                for item in ('expires', 'session_key', 'ss', 'user')] +
                [self._secret],
            )).hexdigest()
            if hash == request.cookies[self._key]:
                # OK, Facebook user is verified: return the user id.
                return {
                    'id': str(request.cookies['%s_user' % self._key]),
                    'token': str(request.cookies['%s_session_key' % \
                        self._key]),
                    'secret': str(request.cookies['%s_ss' % self._key]),
                }

    def post(self, text, uid, **kw):
        """Post the message to the user's facebook profile"""

        self._api.stream.publish(uid=uid, message=text, **kw)

    def __getattr__(self, attr):
        """Delegate the rest of the calls to the API."""

        return getattr(self._api, attr)



import cgi

from openid import extension, message
from openid.consumer import consumer
from openid.extensions import ax
from openid.store import memstore


AX_NS = 'http://openid.net/srv/ax/1.0'


class GoogleClient(object):
    """Holds the settings for Google-specific OpenID."""

    # Required information for Attribute Exchange
    required = {
        'firstname': 'http://axschema.org/namePerson/first',
        'lastname': 'http://axschema.org/namePerson/last',
        'email': 'http://axschema.org/contact/email',
    }

    _default_config = {
        'endpoint': 'https://www.google.com/accounts/o8/id',
    }

    def __init__(self, config={}):
        """Configure the client."""

        for key, value in self._default_config.iteritems():
            if key not in config:
                config[key] = value

        self._config = config

    def redirect(self):
        """Return a custom auth request url."""

        gconsumer = consumer.Consumer({}, memstore.MemoryStore())

        # Create an auth request to the given endpoint
        authrequest = gconsumer.begin(self._config['endpoint'])

        # Create an Attribute Exchange extension request
        axrequest = ax.FetchRequest(self._config['callback'])
        for alias, url in self.required.iteritems():
            axrequest.add(ax.AttrInfo(url, alias=alias, required=True))
        authrequest.addExtension(axrequest)

        # Create and apply a custom User Interface extension request
        uirequest = UIFetchRequest(mode='popup', icon=True)
        authrequest.addExtension(uirequest)

        # Return the generated URL
        return authrequest.redirectURL(self._config['realm'],
            self._config['callback'])

    def get_user(self, query, callback):
        """Verify the request and returns the user's credentials on success."""

        # We use the GenericConsumer class, so we need to add the nonce:
        if 'janrain_nonce' not in query:
            return None

        gconsumer = consumer.Consumer({}, memstore.MemoryStore())
        return_to = '%s?janrain_nonce=%s' % (callback, query['janrain_nonce'])
        response = gconsumer.complete(query, return_to)

        if response.status == consumer.SUCCESS:
            url = response.message.getArg(message.OPENID2_NS, 'claimed_id')
            email, first_name, last_name = (response.message.getArg(AX_NS,
                'value.%s' % val) for val in sorted(self.required.keys()))

            return {
                'id': cgi.parse_qs(url.split('?', 1)[-1])['id'][0],
                'first_name': first_name,
                'last_name': last_name,
                'email': email,
            }


class UIFetchRequest(extension.Extension):
    """User Interface extension for OpenID."""

    ns_alias = 'ui'
    ns_uri = 'http://specs.openid.net/extensions/ui/1.0'

    def __init__(self, mode=None, icon=False, x_has_session=False):
        """Some custom UI initialization."""

        super(UIFetchRequest, self).__init__()

        self._args = {}

        mode is not None and self._args.update({'mode': mode})
        icon and self._args.update({'icon': str(icon).lower()})
        x_has_session and self._args.update(
            {'x-has-session': str(x_has_session).lower()})

    def getExtensionArgs(self):
        """Return extension arguments."""

        return self._args



import doctest
import unittest


docfiles = ('middleware.rst', 'django.rst')


def test_suite():
    """Run all doctests in one test suite."""

    tests = [doctest.DocFileSuite(file,
        optionflags=doctest.ELLIPSIS) for file in docfiles]

    return unittest.TestSuite(tests)



import ao.shorturl

from google.appengine.api import memcache
from google.appengine.ext import db


class ShortUrl(db.Model):
    """Stores the short URL tokens in the datastore."""

    context = db.ReferenceProperty(collection_name='shorturl')


class AppEngineShortUrlHandler(ao.shorturl.BaseShortUrlHandler):
    """Short URL handler for Google App Engine.

    Uses the Datastore to store URLs and tokens, and the Memcache API for
    caching tokens.

    """

    def cache_context(self, url, context):
        """Store the context's key name and kind in memcache."""

        memcache.add(url, str(context.key()), self.url_cache_time)

    def get_context_from_cache(self, url):
        """Get the token from memcache."""

        key = memcache.get(url)
        if key is None:
            raise LookupError('Context key not found in the cache.')

        return db.get(key)

    def get_context_from_db(self, url):
        """Get the context from the datastore."""

        mapping = ShortUrl.get_by_key_name(url)
        if mapping is None:
            raise LookupError('Context not found in the datastore.')

        return mapping.context

    def assign_url(self, context):
        """Create a new URL for the context and assign it to the context."""

        url = ShortUrl(key_name=self.generate_url(), context=context)
        url.put()

    def construct_url(self, context, request=None):
        """Construct the short url for the given context."""

        if context.shorturl.count() == 0:
            self.assign_url(context)  # create a new short url

        return self.url_pattern(context.shorturl[0].key().name())



import os

from setuptools import find_packages, setup


setup(
    # Package information:
    name='ao.social',
    version='1.0.2',
    license='GNU GPL',
    url='http://github.com/aatiis/ao.social',
    description='Social networking middleware for Facebook, Twitter, LinkedIn'\
    ' and Google (OpenID + Buzz)',
    long_description=\
        open('README.rst').read() + \
        open('TODO.rst').read() + \
        open('CHANGES.rst').read(),
    # Author information:
    author='Attila Olah',
    author_email='attilaolah@gmail.com',
    # Package settings:
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=('ao',),
    include_package_data=True,
    zip_safe=True,
    install_requires=(
        'beaker',
        'oauth',
        'pyfacebook',
        'python-openid >= 2.2.5',
    ),
    tests_require=(
        'Django >= 1.1.1',
        'minimock',
        'PyYAML',
        'simplejson',
        'WebTest',
        'zope.testing',
    ),
    extras_require={
        'test': (
            'Django >= 1.1.1',
            'minimock',
            'python-memcached',
            'PyYAML',
            'simplejson',
            'WebTest',
            'zope.testing',
        ),
        'docs': (
            'Sphinx',
            'z3c.recipe.sphinxdoc',
        ),
    },
    # Classifiers:
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Buildout',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.5',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
    ],
)



import urllib

from ao import social

from hashlib import md5

from django import template


register = template.Library()


def apikey(parser, token):
    """Renders the API key."""

    method = token.split_contents()[1]

    if method not in ('facebook', 'twitter', 'linkedin'):
        raise template.TemplateSyntaxError('Unsupported method for `apikey`: '\
            '%r' % method)

    return APIKey(method)


register.tag(apikey)


class APIKey(template.Node):
    """The API key node."""

    def __init__(self, method):
        """Save the login method to self.method."""

        self.method = method

    def render(self, context):
        """Render a button for the given context."""

        return social.getClient(self.method).key()


def loginbutton(parser, token):
    """Renders a login button."""

    token = token.split_contents()

    method = token[1]
    onlogin = len(token) > 2 and token[2] or None

    return LoginButton(method, onlogin)


register.tag(loginbutton)


class LoginButton(template.Node):
    """The Login Button node."""

    def __init__(self, method, onlogin=None):
        """Save the login method to self.method."""

        self.method = method
        self.onlogin = onlogin

    def render(self, context):
        """Render a button for the given context."""

        environ = context['request'].environ
        postlogin = 'opener.location.href=\'%s\'' % \
            urllib.quote(environ['PATH_INFO'])

        if self.method == 'facebook':
            if self.onlogin is not None:
                return '<fb:login-button onlogin="%s"></fb:login-button>' % \
                    self.onlogin
            button = '<fb:login-button onlogin="location.href=\'%s\'">'\
                '</fb:login-button>'
            postlogin = 'location.href=\\\'%s\\\'' % \
                urllib.quote(environ['PATH_INFO'])
        elif self.method == 'twitter':
            button = '<a href="%s" class="login login-twitter popup">&nbsp;'\
                '</a>'
        elif self.method == 'google':
            button = '<a href="%s" class="login login-google popup">&nbsp;</a>'

        auth_url = environ['ao.social.login'] % self.method
        auth_url += '?postlogin=' + postlogin

        return button % auth_url


def avatar(parser, token):
    """Renders the user's avatar."""

    width, height = token.split_contents()[1:3]

    return Avatar(width, height)

register.tag(avatar)


class Avatar(template.Node):
    """The Avatar node."""

    def __init__(self, width, height):
        """Save width and height to self.width and self.height."""

        self.width, self.height = width, height

    def gravatar(email, size=120):
        """Constructs a Gravatar image URL."""

        return 'http://www.gravatar.com/avatar.php?%s' % urllib.urlencode({
            'gravatar_id': md5((email or '').lower()).hexdigest(),
            'size': str(size),
        })

    gravatar = staticmethod(gravatar)

    def render(self, context):
        """Render the avatar for the given context."""

        user = context['request'].environ['ao.social.user']

        url = user.avatar or self.gravatar(user.email,
            max(map(int, (self.width, self.height))))

        img = '<img src="%s" alt="%s" style="width: %spx; height: %spx;"/>'
        img = img % (url, user.name, self.width, self.height)

        return img
